/**
 * Genau 1 - Manual Plotter
 * An example of interactive use of the Control class.
 *
 * Note:
 * The position coordinates are expressed in motor steps: 
 * 80 steps = 1mm
 *
 * Keys:
 * SPACEBAR    Raise/lower the pen
 * ENTER       Move the Axidraw to the location of the mouse
 * LEFT MOUSE  Add a waypoint
 * p           Begin drawing bewteen all waypoints, then clear them
 * SHIFT + P   Clear all waypoints
 * t           Axidraw will begin to follow the mouse [!Warning!] this function doesn't work particularly well.
 *             
 */

Control c;
int ratio;
int tempX;
int tempY;
boolean clear = false;
boolean mouseToggle = false;
ArrayList<waypoint> points = new ArrayList<waypoint>();

void setup() {
  size(420, 297);
  ratio = 80;
  
  c = new Control(this);  // Initialize the control class
  Serial p = c.open();    // Open the serial port
  if (p == null) {  
    println("Axidraw not found.");
    exit();               // No AxiDraw - no joy!
    return;
  } 
  c.readPos();            // Read out the steps from the EBB, set internal pos[] accordingly;
                          // this makes sure that the position is updated when re-launching the program
                          // so a "reset" (via zero()) is not needed
  c.up(true);             // Force the pen to be "up" 
  
  c.move(ratio, 0);
}

void draw() {

  // c.version();
  // c.querysteps();      // Uncomment for some extra info in the console
  // c.queryMotor();
  // c.queryPen();

  messageLoop(c.port); 

  String out = "";
  out += "pos[]: " + c.x() + "," + c.y() + " (steps)\n";
  out += "time: " + millis() + "ms\n";
  out += "idle: " + (c.idle()) + "\n";
  out += "headPos: " + (c.x()+ ", " + c.y()) + "\n";
  

  if (!c.enabled()) {
    out += "\nManually move the pen\nto the top left corner... \n\nPress F1 again when done.";
    background(220, 60, 60);
  } else {
    background(220);
  }
  
  if (clear == true){  //removes the waypoints
    points.clear();
    clear = false;
  }

  fill(0, 140);
  text(out, 30, 30);
  
  stroke(1);
  line(0, mouseY, width, mouseY);                      // Draws the horizontal and vertical lines around the cursor
  line(mouseX, 0, mouseX, height);
  text(mouseX + ", "+ mouseY, mouseX+10, mouseY-10);  // displays the coordinates of the cursor
  
  
  for(int i = 0; i < points.size(); i++){        //Displays waypoints and paths to be drawn after pressing 'p'
    points.get(i).display();
    stroke(1);
    if (i+1 == points.size()){
      return;
    }else{
      line(points.get(i).X, points.get(i).Y, points.get(i+1).X, points.get(i+1).Y);
    }
  }
  
  if (mouseToggle == true && millis()%5 == 1){    //rudimentary mouse-following code
    tempX = int(lerp(mouseX*ratio, c.x(), 0.1));  // Doesn't work very well as the Axidraw pauses other processes while it is moving
    tempY = int(lerp(mouseY*ratio, c.y(), 0.1));  // This Lerp function calculates a point between two other points, eg. 10% of the distance from the position of the mouse and the poisition of the Axidraw Head
    c.moveTo(tempX, tempY);
  }
}

int steps = 800;             // 800 steps = 1cm

void keyPressed() {
  
  if (!c.idle()) return;
   
  if ( key == ' ') {         // Toggles the pen position
    if (c.pen() == Control.DOWN) {
      c.up();
    } else {
      c.down();
    }
  }else if(key == ENTER) { //move head to cursor location (a4 size)
    int time = 0;
    int newX = (mouseX*ratio)- c.x();
    int newY = (mouseY*ratio) - c.y();
    time += c.move(newX, newY);
    //time += c.move(ratio, ratio);
    
    print(time + "units");

  }else if (key == 'p'){  // Begins drawing between each waypoint chosen
    c.up();               // Starts and ends by going to 0,0
    c.moveTo(0,0);
    c.moveTo(int(points.get(0).Xr), int(points.get(0).Yr));
    c.down();
    for(int i = 0; i < points.size(); i++){
      c.moveTo(int(points.get(i).Xr), int(points.get(i).Yr));
    }
    c.up();
    c.moveTo(0,0);
    clear = true;
    
    
  }else if (key == 'P'){ // Clears all waypoints
    points.clear();
    
    
  }else if (key == 't'){ //Begins mouse-following
    mouseToggle = !mouseToggle;
  }
  
  
}

void mousePressed(){ //click on the screen to add a waypoint
  points.add(new waypoint(mouseX, mouseY));
  
  
}
